<?php
/************************************************************
DBF to SQL helper class v0.1  by Nick Vrtis
vrtis_at_vrtisworks_dot_com

Input		: name of the DBF( dBase III plus) file
Output	:	- dbf_num_rec, the number of records
			- dbf_num_field, the number of fields
			- dbf_names, array of field information ('name', 'len', 'type')

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU  Lesser General Public License for more details.
  
**************************************************************/ 
class dbf2sql_class {

    var $_dbf;
    var $_sqltable;
    var $_sqlbeg;

    function dbf2sql_class($dbf, $sqltable, $miva=false) {
        $this->_dbf=$dbf;
        $this->_sqltable=$sqltable;
        if ($miva) {    //We will "bit fiddle" a few things for some Miva files
            if ($sqltable=='orders_dbf') {
                //For the orders db.. the date is really a timestamp...
                $idx = $this->_dbf->dbf_nameidx['DATE'];
                $this->_dbf->dbf_names[$idx]['type']='T';
                //And total is a dollar field
                $idx = $this->_dbf->dbf_nameidx['TOTAL'];
                $this->_dbf->dbf_names[$idx]['type']='$';
            } elseif ($sqltable=='orditems_dbf') {
                //Price is dollar field
                $idx = $this->_dbf->dbf_nameidx['PRICE'];
                $this->_dbf->dbf_names[$idx]['type']='$';
                //Weight is a fraction....
                $idx = $this->_dbf->dbf_nameidx['WEIGHT'];
                $this->_dbf->dbf_names[$idx]['type']='F';
            } elseif ($sqltable=='products_dbf') {
                //Price is dollar field
                $idx = $this->_dbf->dbf_nameidx['PRICE'];
                $this->_dbf->dbf_names[$idx]['type']='$';
                //Cost is dollar field
                $idx = $this->_dbf->dbf_nameidx['COST'];
                $this->_dbf->dbf_names[$idx]['type']='$';
                //Weight is a fraction....
                $idx = $this->_dbf->dbf_nameidx['WEIGHT'];
                $this->_dbf->dbf_names[$idx]['type']='F';
            } elseif ($sqltable=='ordrchrg_dbf') {
                //Amount is dollar field
                $idx = $this->_dbf->dbf_nameidx['AMOUNT'];
                $this->_dbf->dbf_names[$idx]['type']='$';
                //Display amount is also a dollar field
                $idx = $this->_dbf->dbf_nameidx['DISP_AMT'];
                $this->_dbf->dbf_names[$idx]['type']='$';
            } elseif ($sqltable=='ordopts_dbf') {
                //Price is dollar field
                $idx = $this->_dbf->dbf_nameidx['PRICE'];
                $this->_dbf->dbf_names[$idx]['type']='$';
            } elseif ($sqltable=='vendororders_dbf') {
                $idx = $this->_dbf->dbf_nameidx['SENT'];
                $this->_dbf->dbf_names[$idx]['type']='T';
                $idx = $this->_dbf->dbf_nameidx['ORD_DATE'];
                $this->_dbf->dbf_names[$idx]['type']='T';
                $idx = $this->_dbf->dbf_nameidx['PAYOUT'];
                $this->_dbf->dbf_names[$idx]['type']='$';
            }
        }
        $this->_sqlbeg = 'INSERT INTO '.$this->_sqltable.' (';    //Create the start of the sql insert...
        //We list all the names in case somebody has added a few extra columns manually..
        for ($i=0; $i<$this->_dbf->dbf_num_field; $i++) {
            $this->_sqlbeg.='`'.$this->_dbf->dbf_names[$i]['name'].'`,';
        }
        //Don't forget to drop the last ,
        $this->_sqlbeg = substr($this->_sqlbeg,0,strlen($this->_sqlbeg)-1).') VALUES (';
    }
    function sqldefine($idx) {
        //Create the SQL variable definations from the dbf column descriptions..
        //NOTE: it does not set up defaults, and does not add a comma after..
        $avar='`'.$this->_dbf->dbf_names[$idx]['name'].'` ';
        if ($this->_dbf->dbf_names[$idx]['type']=='N') {
            $avar.='int('.$this->_dbf->dbf_names[$idx]['len'].')';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='C') {
            $avar.='varchar('.$this->_dbf->dbf_names[$idx]['len'].')';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='D') {
            $avar.='date';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='M') {
            $avar.='text';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='G'
            || $this->_dbf->dbf_names[$idx]['type']=='B'
            || $this->_dbf->dbf_names[$idx]['type']=='P') {
            $avar.='blob';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='L') {
            $avar.='char(1)';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='T') {
            $avar.='datetime';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='$') {
            $avar.='decimal('.$this->_dbf->dbf_names[$idx]['len'].',2)';
        } elseif ($this->_dbf->dbf_names[$idx]['type']=='F') {
            $avar.='float';
        } else {
            $avar.='***Unknown type='.$this->_dbf->dbf_names[$idx]['type'].'***';
        }
        return $avar;
    }
    function sqlinsert($row){
        $sql = '';
        for ($j=0; $j<$this->_dbf->dbf_num_field; $j++) {
            if ($this->_dbf->dbf_names[$j]['type']=='N' ||
                $this->_dbf->dbf_names[$j]['type']=='$' ||
                $this->_dbf->dbf_names[$j]['type']=='F') {
                if (strlen($row[$j])==0) {
                    $val=0;
                } else {
                    $val=$row[$j];
                }
                $sql.=$val.',';
            } elseif ($this->_dbf->dbf_names[$j]['type']=='C') {
                $sql.="'".mysql_escape_string($row[$j])."',";
            } elseif ($this->_dbf->dbf_names[$j]['type']=='M') {
                $sql.="'".mysql_escape_string($row[$j])."',";
            } elseif ($this->_dbf->dbf_names[$j]['type']=='L') {
                $sql.="'".mysql_escape_string($row[$j])."',";
            } elseif ($this->_dbf->dbf_names[$j]['type']=='D') {
                if (strlen($row[$j])==0) {
                   $dt='0000-00-00';
                } else {
                   $dt=$row[$j];
                }
                $sql.=$dt.',';
            } elseif ($this->_dbf->dbf_names[$j]['type']=='T') {
                if (strlen($row[$j])==0) {
                   $dt="'0000-00-00 00:00:00'";
                } else {
                   $dt=date("'Y-m-d H:i:s'",$row[$j]+0);
                }
                $sql.=$dt.',';
            }   //For now.. I am going to ignore blobs....
        }
        $sql=$this->_sqlbeg.substr($sql,0,strlen($sql)-1).')';
        //$got=0;
        $got = mysql_query($sql);
        if(!$got) {
            echo '<h2>Insert failed:<br/>'.$sql.'<br>'.mysql_error().'</h2><br>';
            return;
        }
    }
}//End of Class
?>