<?php
/************************************************************
DBF reader Class v0.04  by Faro K Rasyid (Orca)
orca75_at_dotgeek_dot_org
v0.05 by Nicholas Vrtis
vrtis_at_vrtisworks_dot_com
1) changed to not read in complete file at creation.
2) added function to read individual rows
3) added support for Memo fields in dbt files.
4) See: http://www.clicketyclick.dk/databases/xbase/format/dbf.html#DBF_STRUCT
   for some additional information on XBase structure...
5) NOTE: the whole file (and the memo file) is read in at once.  So this could
   take a lot of memory for large files.
v0.06 by Nicholas Vrtis
1) Added support for Foxpro FPT type memo files.  Note: Foxpro memo files can have
binary image data stored in them.. so be careful
2) Added an Optional "inmensize" parameter to the class creation function.  If the
main file plus the memo file (if present) are less than this, the files will not
be read in all at once, but only when records are requested.  In 100,000 increments
(1=100,000.. 2=200,000... etc.).  Use 0 to never read in the whole file, use <0
to always read in a file.. regardless of size.
v0.07 by Nicholas Vrtis
1) Removed getRowAssoc function.
2) Made the $assoc actually work
3) Added $dbf_file_name and $dbf_nameidx

Input		: name of the DBF( dBase III plus) file
Output	:	- dbf_num_rec, the number of records
			- dbf_num_field, the number of fields
			- dbf_names, array of field information ('name', 'len', 'type')

Usage	example:
$file= "your_file.dbf";//WARNING !!! CASE SENSITIVE APPLIED !!!!!
$dbf = new dbf_class($file);
$num_rec=$dbf->dbf_num_rec;
$num_field=$dbf->dbf_num_field;

for($i=0; $i<$num_rec; $i++){
    $row = $dbf->getRow($i);
	for($j=0; $j<$num_field; $j++){
		echo $row[$j].' ');
	}
	echo('<br>');
}

Thanks to :
- Willy
- Miryadi

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU  Lesser General Public License for more details.
  
**************************************************************/ 
class dbf_class {
		
    var $dbf_num_rec;           //Number of records in the file
    var $dbf_num_field;         //Number of columns in each row
    var $dbf_names = array();   //Information on each column ['name'],['len'],['type']
    var $dbf_file_type;			//Type of Xbase file this is...
    var $dbf_file_name;         //The name of the file..
    var $dbf_nameidx = array(); //An associative array key='name' value=index to $dbf_names and $row
    //These are private....
    var $_raw;           		//The raw input file
    var $_rowsize;          	//Length of each row
    var $_hdrsize;          	//Length of the header information (offset to 1st record)
    var $_memos;             	//The raw memo file (if there is one).
    var $_memohdr=512;		 	//Length of the memo file header
    var $_memoblk;			 	//Length of each block of the memo file
    var $_dbfhandle;            //File handle if $_allin=FALSE;
    var $_dbthandle;            //File handle for the memo file....
    var $_allin=true;           //==TRUE means whole file is already read in
                                //==FALSE means each record needs to be read when requested.
    var $_memoeot;              //EOT characters for a memo...

    function dbf_class($filename, $inmemsize=1) {
        $this->_memoeot=chr(26).chr(26);
        if ( !file_exists($filename)) {
            echo $filename.' DBF file does not exist!'; exit;
        }
        $tail=substr($filename,-4);
        if (strcasecmp($tail, '.dbf')!=0) {
            echo 'Not a valid DBF file !!'; exit;
        }
				
        $filesize = filesize($filename);
        $this->dbf_file_name=$filename;     //Save in case somebody needs to know...
        //Initial information
        $infohdr = 32;//Header Size
        $this->_dbfhandle = fopen($filename, "r");
        if (!$this->_dbfhandle) { echo "Cannot read DBF file"; exit; }
        $hdr = fread ($this->_dbfhandle, $infohdr);
        //Make sure that we indeed have a dbf file...
        //See http://www.clicketyclick.dk/databases/xbase/format/dbf.html
        // 3= file without DBT memo file; 131 ($83)= file with a DBT. 245 ($F5)= file with a FPT
        $this->dbf_file_type = ord($hdr[0]);
        if(!($this->dbf_file_type==3 || $this->dbf_file_type == 131 ||
            $this->dbf_file_type == 245)) {
            echo 'Not a valid DBF file !!!'; exit;
        }
        //Number of records
        $this->dbf_num_rec = ord($hdr[7])*16777216+ord($hdr[6])*65536+ord($hdr[5])*256+ord($hdr[4]);
        //Total size of header+column descriptions.
        $this->_hdrsize=ord($hdr[9])*256+ord($hdr[8]);
        //Size of each row data
        $this->_rowsize = ord($hdr[11])*256+ord($hdr[10]);
        //Number of fields
		$this->dbf_num_field = floor(($this->_hdrsize - $infohdr ) / $infohdr ) ;//Number of Fields
        //Now read in the whole header to get the column definations
        rewind($this->_dbfhandle);
        $hdr = fread ($this->_dbfhandle, $this->_hdrsize);
				
        //Field properties retrieval looping
        for($j=0; $j<$this->dbf_num_field; $j++){
            $name = '';
            $beg = $j*$infohdr+$infohdr;
            for($k=$beg; $k<$beg+11; $k++){
                if(ord($hdr[$k])==0){
                    break;  //quit on the trailing 0
                }
                $name .= $hdr[$k];
            }
            $this->dbf_names[$j]['name']= $name;//Name of the Field
            $this->dbf_names[$j]['len']= ord($hdr[$beg+16]);//Length of the field
            $this->dbf_names[$j]['type']= $hdr[$beg+11];
            $this->dbf_nameidx[$name]=$j;       //Makes looking up info by name easier..
        }
        if ($this->dbf_file_type==131 || 
        	$this->dbf_file_type == 245) { //See if this has a memo file with it...
            //Read the File
            $tail=substr($tail,-1,1);   //Get the last character...
            if ($this->dbf_file_type==245) {
	            if ($tail=='F'){            //See if upper or lower case
	                $tail='FPT';              //Keep the case the same
	            } else {
	                $tail='fpt';
	            }
	            $memoname = substr($filename,0,strlen($filename)-3).$tail;
            } else {
	            if ($tail=='F'){            //See if upper or lower case
	                $tail='T';              //Keep the case the same
	            } else {
	                $tail='t';
	            }
	            $memoname = substr($filename,0,strlen($filename)-1).$tail;
            }
            $this->_dbthandle = fopen($memoname, "r");
            if (!$this->_dbthandle) { echo "Cannot read DBT file $memoname"; exit; }
            if ($this->dbf_file_type==245) {
                $this->_memos=fread($this->_dbthandle, 8);  //Need read the block size...
            	$this->_memoblk=ord($this->_memos[6])*256+ord($this->_memos[7]);
           	} else {
           		$this->_memoblk=512;
           	}
            $memosize = filesize($memoname);
        } else {
            $memosize=0;    //No memo file size to account for
        }
        if ($inmemsize<0) {
            $this->_allin=true;     //Read in regardless of file sizes...
        } elseif ($inmemsize==0) {
            $this->_allin=false;    //Do NOT read in regardless of file sizes
        } else {
            $filesize = filesize($filename);    //Get file size of the dbf
            $fsize = $memosize+$filesize;       //Combine both...
            if ($fsize<($inmemsize*100000)) {   //See if within all in range....
                $this->_allin=true;             //YES-do the read now....
            } else {
                $this->_allin=false;            //NO-do not read them in...
            }
        }
        if ($this->_allin) {
            //Now we need to actually read in the files...
            rewind($this->_dbfhandle);
            $this->_raw = fread($this->_dbfhandle, $filesize);
            fclose ($this->_dbfhandle);
            if ($this->dbf_file_type==131 || 
            	$this->dbf_file_type == 245) { //See if this has a memo file with it...
                rewind($this->_dbthandle);
                $this->_memos = fread($this->_dbthandle, $memosize);
                fclose ($this->_dbthandle);
            }
        }
        //NOTE: we leave the files open so we do not have to go through open/close for each read...
    }
    
    function getRow($recnum, $assoc=false) {
        if ($recnum>=$this->dbf_num_rec) {
            return false;
        }
        if ($this->_allin) {
            $rawrow = substr($this->_raw,$recnum*$this->_rowsize+$this->_hdrsize,$this->_rowsize);
        } else {
            fseek($this->_dbfhandle, $recnum*$this->_rowsize+$this->_hdrsize);
            $rawrow = fread($this->_dbfhandle, $this->_rowsize);
        }
        $rowrecs = array();
        $beg=1;
        if (ord($rawrow[0])==42) {
            return false;   //Record is deleted...
        }
        for ($i=0; $i<$this->dbf_num_field; $i++) {
            if ($assoc) {
                $name=$this->dbf_names[$i]['name'];            
            }
            $col=trim(substr($rawrow,$beg,$this->dbf_names[$i]['len']));
            if ($this->dbf_names[$i]['type']=='M'
            	|| $this->dbf_names[$i]['type']=='G'
            	|| $this->dbf_names[$i]['type']=='P'
            	|| $this->dbf_names[$i]['type']=='B') {
            	if ($this->dbf_file_type == 245) {
            		//FPT are complicated...
            		if (strlen($col)!=0) {
            			$memobeg=$col*$this->_memoblk;
                        if ($this->_allin) {
                            //Just need to substring the record out of memory
                			$memolen=ord($this->_memos[$memobeg+4])*16777216+
                				ord($this->_memos[$memobeg+5])*65536+
                				ord($this->_memos[$memobeg+6])*256+
                				ord($this->_memos[$memobeg+7]);
            	    		$memobeg+=8;
                            if ($assoc) {
                			    $rowrecs[$name]=substr($this->_memos,$memobeg,$memolen);
                            } else {
                			    $rowrecs[]=substr($this->_memos,$memobeg,$memolen);
                            }
                        } else {
                            fseek($this->_dbthandle, $memobeg);
                            //Read in the header to get the length....
                            $this->_memos = fread($this->_dbthandle, 8);
                			$memolen=ord($this->_memos[4])*16777216+
                				ord($this->_memos[5])*65536+
                				ord($this->_memos[6])*256+
                				ord($this->_memos[7]);
                            //Note--we don't need to fseek.. because it starts right after the header...
                            if ($assoc) {
                                $rowrecs[$name] = fread($this->_dbthandle, $memolen);
                            } else {
                                $rowrecs[] = fread($this->_dbthandle, $memolen);
                            }
                            /* echo "READ r=$recnum; b=$memobeg; l=$memolen; t=".$this->dbf_names[$i]['type']." blk=".
                                $this->_memoblk."<br>"; */
                        }
            		} else {
                        if ($assoc) {
                			$rowrecs[$name]='';
                        } else {
                			$rowrecs[]='';
                        }
            		}
            	} else {
            		//DBT files are relatively easy....
                    if (strlen($col)!=0) {
	                    $memobeg=$col*$this->_memoblk;  //Find start of the memo block (0=header so it works)
                        if ($this->_allin) {
	                        $memoend=strpos($this->_memos,$this->_memoeot,$memobeg);   //Find the end of the memo
                            if ($assoc) {
	                            $rowrecs[$name]=substr($this->_memos,$memobeg,$memoend-$memobeg);
                            } else {
	                            $rowrecs[]=substr($this->_memos,$memobeg,$memoend-$memobeg);
                                //$rowrecs[]='Beg='.$memobeg.' End='.$memoend;
                            }
                        } else {
                            fseek($this->_dbthandle, $memobeg);
                            $this->_memos=fread($this->_dbthandle, $this->_memoblk);
                            //We need to read in blocks until we find the end of the memo field...
                            while (($memoend=strpos($this->_memos, $this->_memoeot))===false) {
                                $this->_memos.=fread($this->_dbthandle, $this->_memoblk);
                            }
                            if ($assoc) {
    	                        $rowrecs[$name]=substr($this->_memos,0,$memoend);
                            } else {
    	                        $rowrecs[]=substr($this->_memos,0,$memoend);
                            }
                        }
            		} else {
                        if ($assoc) {
                			$rowrecs[$name]='';
                        } else {
                			$rowrecs[]='';
                        }
                    }
            	}
            } else {
                if ($assoc) {
                    $rowrecs[$name]=$col;
                } else {
                    $rowrecs[]=$col;
                }
            }
            $beg+=$this->dbf_names[$i]['len'];
        }
        return $rowrecs;
    }
    
    function close() {
        if (!$this->_allin) {
            fclose($this->_dbfhandle);
            if ($this->dbf_file_type==131 || 
            	$this->dbf_file_type == 245) { //See if this has a memo file with it...
                fclose($this->_dbthandle);
            }
        }
        unset($this->_raw);
        unset($this->_memos);
    }
}//End of Class
?>