<?php
/************************************************************
Read a Miva indexed file via the mvx indexs.
v2.0 by Nicholas Vrtis
vrtis_at_vrtisworks_dot_com

Input		: name of the mvx (Miva index) file
Output	:	- record number of the dbf file

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU  Lesser General Public License for more details.
  
**************************************************************/ 
class mvx_class {
		
    var $mvx_file_name;         //The name of the index file...

    //These are private....
    var $raw;           		//The raw input file
    var $blksize;          	    //Length of each index block
    var $keysize;          	    //size of each key+header
    var $root;               	//address of root node
    var $mvxhandle;             //File handle if $allin=FALSE;
    var $thekey;                //Current key we are looking for...
    var $keylen;                //Length of the current key...
    var $keyisnbr;              //True-we are searching for a numeric key
    var $allin=true;            //==TRUE means whole file is already read in
                                //==FALSE means each record needs to be read when requested.
    //There is a breadcrumb array for each level in the tree
    var $breadcrumbs;           //Each breadcrumb represents a block on a particular level
                                //0=starting address of the node
                                //1=current address of the key field
                                //2=address of 'next node' pointer past last valid key field
    var $bclevel;               //Current tree level
    var $bcidx;                 //Current idx on this level

    function mvx_class($mvxname, $inmemsize=1) {
        if ( !file_exists($mvxname)) {
            echo $mvxname.' MVX file does not exist!'; exit;
        }
        $tail=substr($mvxname,-4);
        if (strcasecmp($tail, '.mvx')!=0) {
            echo 'Not a valid mvx file !!'; exit;
        }
				
        $filesize = filesize($mvxname);
        $this->mvx_file_name=$mvxname;     //Save in case somebody needs to know...
        //Initial information
        $infohdr = 30;//Header Size
        $this->mvxhandle = fopen($mvxname, "r");
        if (!$this->mvxhandle) { echo "Cannot read MVX file"; exit; }
        $hdr = fread ($this->mvxhandle, $infohdr);
        //Make sure that we indeed have a mvx file...
        //See http://mivo.truxoft.com/art0027.htm
        if(strncasecmp('HTSI',$hdr,4)!=0) {
            echo 'Not a valid MVX file !!!'; exit;
        }
        //Size of index block
        $this->blksize = ord($hdr[20])*256+ord($hdr[19]);
        //Size of each key
        $this->keysize = ord($hdr[22])*256+ord($hdr[21]);
        //Root node
        $this->root = (ord($hdr[16])*16777216+ord($hdr[15])*65536+ord($hdr[14])*256+ord($hdr[13]))
            * $this->blksize;
        if ($inmemsize<0) {
            $this->allin=true;     //Read in regardless of file sizes...
        } elseif ($inmemsize==0) {
            $this->allin=false;    //Do NOT read in regardless of file sizes
        } else {
            $filesize = filesize($mvxname);    //Get file size of the dbf
            if ($filesize<($inmemsize*100000)) {   //See if within all in range....
                $this->allin=true;             //YES-do the read now....
            } else {
                $this->allin=false;            //NO-do not read them in...
            }
        }
        if ($this->allin) {
            //Now we need to actually read in the files...
            rewind($this->mvxhandle);
            $this->raw = fread($this->mvxhandle, $filesize);
            fclose ($this->mvxhandle);
        }
        //NOTE: we leave the files open so we do not have to go through open/close for each read...
        //echo 'Root=',sprintf('%X',$this->root),' Blksize=',sprintf('%X',$this->blksize),'<br>';
    }
    
    function findKey($key, $asnbr=false) {
        $this->theKey=$key;
        $this->keylen=strlen($this->theKey);
        $this->keyisnbr=$asnbr;
        $this->gotoRoot();     //start looking from the root node...
        $x=$this->searchKey();
        if ($x!==false) {
            $x=$this->getDbfRecNbr();
        }
        return $x;
    }
    //This is recursive...  search for the lowest occurance of the key
    function searchKey() {
        $x=$this->rtnCmp(); //See where we are...
        while ($x>0) {
            $x=$this->right();  //Go right until we find a db key >=
        }
        if ($this->down()===false) {
            //Can't go down--where are we....
            $x=$this->rtnCmp();
            if ($x<=0) {
                return $x;          //Either found it.. or it wasn't there
            }
            //We never return an equal if the is a lower branch available
            // since there might be equals down that branch...
            //So we need to go up 1 level and see if that was an equal we missed.
            $x=$this->upLevel();    //Still less.. might be higher up the tree
            if ($x===false || $x<=0) {
                return $x;          //Either we found it here, or it doesn't exist.
            }
        }
        return $this->searchKey();
    }
    function upLevel() {
        if ($this->bclevel==0) {
            return false;       //Can't go up further
        }
        $this->bclevel--;
        //We always came from the last entry for that level..
        $this->bcidx=count($this->breadcrumbs[$this->bclevel])-1;
        if (!$this->allin) {
            //Need to read the block back in..
            fseek($this->mvxhandle, $this->breadcrumbs[$this->bclevel][$this->bcidx][0]);
            $this->raw = fread($this->mvxhandle, $this->blksize);
        }
        $this->dbgout('Ul');
        return $this->rtnCmp();
    }
    //Advances to the next entry 'to the right' on the same level
    //NOTE: 'right' is different than next in key sequence.
    function right() {
        $rawadr=$this->breadcrumbs[$this->bclevel][$this->bcidx][1]+$this->keysize; //Just advance within the block
        if ($rawadr<$this->breadcrumbs[$this->bclevel][$this->bcidx][2]) {
            //We are within the current block, so it is pretty simple...
            $this->breadcrumbs[$this->bclevel][$this->bcidx][1]=$rawadr;   //Save new current pointer
            $this->dbgout('Ri');
            return $this->rtnCmp();
        }
        $newbc=$this->readBlock($rawadr);
        if (!$newbc) {
            //Couldn't go any further.. end of this level
            $this->dbgout('Re');
            return false;
        }
        $this->bcidx++; //We are advancing within the level
        $this->breadcrumbs[$this->bclevel][$this->bcidx]=$newbc;
        $this->dbgout('Rl');
        return $this->rtnCmp();
    }
    //Goes down one level to the first entry on the new level
    function down() {
        $rawadr=$this->breadcrumbs[$this->bclevel][$this->bcidx][1];
        $newbc=$this->readBlock($rawadr);
        if (!$newbc) {
            //Couldn't go down..
            $this->dbgout('De');
            return false;
        }
        $this->bclevel++;   //We are advancing down a level
        $this->bcidx=0;     //And this is the first block on that level
        $this->breadcrumbs[$this->bclevel]=array($newbc);
        $this->dbgout('Dl');
        return $this->rtnCmp();
    }
    //Reads (if necessary) the next block - Returns a breadcrumb entry array
    function readBlock($rawadr) {
        $adr = (ord($this->raw[$rawadr+3])*16777216+ord($this->raw[$rawadr+2])*65536+
            ord($this->raw[$rawadr+1])*256+ord($this->raw[$rawadr]))
            * $this->blksize;
        if ($adr==0) {
            return false;
        }
        if ($this->allin) {
            //No need to read it in...
            $ladr=4+$adr+$this->keysize*ord($this->raw[$adr]);
            $cadr=$adr+4;
        } else {
            fseek($this->mvxhandle, $adr);
            $this->raw = fread($this->mvxhandle, $this->blksize);
            $ladr=4+$this->keysize*ord($this->raw[0]);
            $cadr=4;
        }
        return array($adr,$cadr,$ladr);
    }
    //Returns the current key value...
    function getCurrentKey() {
        $rawadr=$this->breadcrumbs[$this->bclevel][$this->bcidx][1]+8;   //Get index of current key position
        $x=strpos($this->raw,0,$rawadr);
        return substr($this->raw,$rawadr,$x-$rawadr);
    }
    function gotoFirst() {
        $this->theKey=chr(0);  //I'm not looking for a key value anyway...
        $this->keylen=1;
        $this->gotoRoot();
        while ($this->down()!==false) {
        }
        return $this->getDbfRecNbr();
    }
    function gotoLast() {
        $this->theKey=chr(255);  //I'm not looking for a key value anyway...
        $this->keylen=1;
        $this->gotoRoot();
        while ($this->right()!==false) {
        }
        return $this->getDbfRecNbr();
    }
    function getNext() {
        $x=$this->right();  //Try to go right once
        if ($x===false) {
            //Can't go any further in the block..
            $x=$this->upLevel();   //bounce up one level
            if ($x!==false) {
                //Return that entry.....
                return $this->getDbfRecNbr();
            }
            //Couldn't go right, and couldn't go up... must be done...
            return false;
        }
        //Need to go find a leaf.....
        while ($this->down()!==false) {
        }
        return $this->getDbfRecNbr();
    }
    function getPrev() {
        die ('getPrev not implemented/tested yet');
        $x=$this->left();  //Try to go left once
        if ($x!==false) {
            //Went left, last in that block is just prior to one we were on...
            $this->lastinBC();
            return $this->getDbfRecNbr();
        }
        //Must be at a leaf (can't go left)
        //Backup one entry in the block...
        while (!$this->previnBC()) {
            //Was at the start of the block.. go up a level
            if ($this->upBC()===false) {
                //Can't go up any further.. 
                return false;
            }
        }
        return $this->getDbfRecNbr();
    }
    function getDbfRecNbr() {
        $x=$this->breadcrumbs[$this->bclevel][$this->bcidx][1];
        return ord($this->raw[$x+7])*16777216+
                   ord($this->raw[$x+6])*65536+
                   ord($this->raw[$x+5])*256+
                   ord($this->raw[$x+4]);
    }
    //Initialize things to the root node...
    function gotoRoot() {
        $this->bclevel=0;
        $this->bcidx=0;
        if ($this->allin) {
            //No need to read it in...
            $ladr=4+$this->root+$this->keysize*ord($this->raw[$this->root]);
            $cadr=$this->root+4;
        } else {
            fseek($this->mvxhandle, $this->root);
            $this->raw = fread($this->mvxhandle, $this->blksize);
            $ladr=4+$this->keysize*ord($this->raw[0]);
            $cadr=4;
        }
        $this->breadcrumbs=array(); //Start with an empty breadcrumb list
        //First entry is the root block
        $this->breadcrumbs[0][0]=array($this->root,$cadr,$ladr,0);
        $this->dbgout('GR');
        return $this->rtnCmp();
    }
    function close() {
        if (!$this->allin) {
            fclose($this->mvxhandle);
        }
        unset($this->raw);
    }
    //Do the comparision to calculcate the return value for the next direction
    function rtnCmp() {
        $adr=$this->breadcrumbs[$this->bclevel][$this->bcidx][1]+8;
        //See if we are looking for numbers (Miva calls it "EXACT"
        if ($this->keyisnbr) {
            $x=$this->keylen-strpos($this->raw,0,$adr)+$adr;
            //echo strpos($this->raw,0,$adr),':',$adr,':',$this->keylen,':',$x,'<br>';
            //For numbers, we need to find the right length first.
            if ($x==0) {
                //If the lengths match.. then the characters count
                $x=strcmp($this->theKey,substr($this->raw,$adr,$this->keylen));
            }
        } else {
            //Not exact, so we just do a compare on the 1st part of the string
            $x=strcmp($this->theKey,substr($this->raw,$adr,$this->keylen));
        }
        return $x;
    }
    function dbgout($dir) {
        if (0==1) {
            $junk=$this->breadcrumbs[$this->bclevel][$this->bcidx];
            $ckl=strpos($this->raw,0,$junk[1]+8)-$junk[1]-8;
            printf('DIR:%s BCLevel:%d BCidx:%d Adr:%8X Cadr:%8X Blen:%8X Key:%s<br>',
                $dir,
                $this->bclevel,
                $this->bcidx,
                $junk[0],
                $junk[1],
                $junk[2],
                substr($this->raw,$junk[1]+8,$ckl));
        }
    }
}//End of Class
?>